==> media_remote_social-1.0.3.tar.gz: media_remote_social/tests/src/Kernel/RemoteSocialErrorHandlingTest.php <==
<?php

namespace Drupal\Tests\media_remote_social\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\media\Entity\Media;
use Drupal\media_remote_social_test\ResourceMiddleware;

/**
 * Tests error handling and edge cases for remote social media.
 *
 * @group media_remote_social
 */
class RemoteSocialErrorHandlingTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'file',
    'image',
    'media',
    'media_remote_social',
    'media_remote_social_test',
    'system',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig('media');
    $this->installConfig('media_remote_social');
    $this->installEntitySchema('file');
    $this->installEntitySchema('media');
    $this->installEntitySchema('user');
    $this->installSchema('file', ['file_usage']);
  }

  /**
   * Tests handling of invalid URLs.
   */
  public function testInvalidUrls(): void {
    $invalid_urls = [
      'not-a-url',
      'https://example.com/not-social-media',
      'https://youtube.com/watch?v=dQw4w9WgXcQ', // Not a supported provider
      'ftp://facebook.com/post/123', // Wrong protocol
      '', // Empty string
    ];

    foreach ($invalid_urls as $url) {
      try {
        $media = Media::create([
          'bundle' => 'remote_social',
          'name' => 'Test Invalid URL',
          'field_media_oembed_social' => $url,
        ]);
        $media->save();
        
        // If we get here without an exception, check that the media is created
        // but the source field validation should prevent invalid URLs.
        $violations = $media->get('field_media_oembed_social')->validate();
        $this->assertGreaterThan(0, $violations->count(), 'Invalid URL should produce validation errors: ' . $url);
      }
      catch (\Exception $e) {
        // Expected behavior for invalid URLs
        $this->assertTrue(TRUE, 'Exception thrown for invalid URL as expected: ' . $url);
      }
    }
  }

  /**
   * Tests handling of malformed oEmbed responses.
   */
  public function testMalformedOembedResponse(): void {
    $url = 'https://www.facebook.com/test/posts/12345/';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    // Create a temporary file with malformed JSON.
    $malformed_json = '{"version": "1.0", "type": "rich", invalid json here}';
    $temp_file = tempnam(sys_get_temp_dir(), 'malformed_oembed');
    file_put_contents($temp_file, $malformed_json);
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => $temp_file,
    ]);
    
    try {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // If save succeeds, the media should still be created but might have limited metadata.
      $this->assertInstanceOf(Media::class, $media);
    }
    catch (\Exception $e) {
      // This is also acceptable behavior.
      $this->assertStringContainsString('oEmbed', $e->getMessage());
    }
    finally {
      unlink($temp_file);
    }
  }

  /**
   * Tests handling of missing required oEmbed fields.
   */
  public function testMissingOembedFields(): void {
    $url = 'https://www.instagram.com/p/TEST123/';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    // Create oEmbed response with missing required fields.
    $incomplete_response = [
      'version' => '1.0',
      'type' => 'rich',
      // Missing: html, width, provider_name, etc.
    ];
    
    $temp_file = tempnam(sys_get_temp_dir(), 'incomplete_oembed');
    file_put_contents($temp_file, json_encode($incomplete_response));
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => $temp_file,
    ]);
    
    try {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // Check that the media was created even with incomplete data.
      $this->assertInstanceOf(Media::class, $media);
      
      // The media name should fall back to a default if title is missing.
      $this->assertNotEmpty($media->getName());
    }
    catch (\Exception $e) {
      // Some missing fields might cause exceptions.
      $this->assertTrue(strpos($e->getMessage(), 'oEmbed') !== FALSE || strpos($e->getMessage(), 'required') !== FALSE);
    }
    finally {
      unlink($temp_file);
    }
  }

  /**
   * Tests that unsupported providers are rejected.
   */
  public function testUnsupportedProviders(): void {
    // These URLs are valid oEmbed URLs but not in our supported list.
    $unsupported_urls = [
      'https://www.youtube.com/watch?v=dQw4w9WgXcQ',
      'https://vimeo.com/123456789',
      'https://soundcloud.com/artist/track',
    ];
    
    foreach ($unsupported_urls as $url) {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      
      // Validate the field.
      $violations = $media->get('field_media_oembed_social')->validate();
      
      // There should be violations for unsupported providers.
      $this->assertGreaterThan(0, $violations->count(), 'Unsupported provider should produce validation errors: ' . $url);
      
      $has_provider_error = FALSE;
      foreach ($violations as $violation) {
        if (strpos($violation->getMessage(), 'provider') !== FALSE || strpos($violation->getMessage(), 'supported') !== FALSE) {
          $has_provider_error = TRUE;
          break;
        }
      }
      
      $this->assertTrue($has_provider_error, 'Validation should mention unsupported provider for: ' . $url);
    }
  }

  /**
   * Tests edge cases with special characters in URLs.
   */
  public function testSpecialCharacterUrls(): void {
    $special_urls = [
      'https://www.facebook.com/test%20user/posts/123456/', // URL encoded spaces
      'https://x.com/user/status/123456789?s=20&t=abc123', // Query parameters
      'https://www.instagram.com/p/ABC-123_DEF/', // Special characters in ID
    ];
    
    foreach ($special_urls as $url) {
      $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
      
      // Use the appropriate fixture based on the provider.
      $provider = 'facebook';
      if (strpos($url, 'x.com') !== FALSE) {
        $provider = 'x';
      }
      elseif (strpos($url, 'instagram.com') !== FALSE) {
        $provider = 'instagram';
      }
      
      $fixture_file = __DIR__ . '/../../fixtures/' . $provider . '.json';
      
      $state = $this->container->get('state');
      $state->set(ResourceMiddleware::class, [
        $resource_url => $fixture_file,
      ]);
      
      $media = Media::create([
        'bundle' => 'remote_social',
        'name' => 'Test Special Characters',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // The media should be created successfully.
      $this->assertInstanceOf(Media::class, $media);
      $this->assertEquals($url, $media->get('field_media_oembed_social')->value);
    }
  }

  /**
   * Tests concurrent media creation with the same URL.
   */
  public function testConcurrentMediaCreation(): void {
    $url = 'https://www.tiktok.com/@user/video/123456789';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => __DIR__ . '/../../fixtures/tiktok.json',
    ]);
    
    // Create multiple media entities with the same URL.
    $media_entities = [];
    for ($i = 0; $i < 3; $i++) {
      $media = Media::create([
        'bundle' => 'remote_social',
        'name' => 'Test TikTok ' . $i,
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      $media_entities[] = $media;
    }
    
    // All media entities should be created successfully.
    $this->assertCount(3, $media_entities);
    
    // Each should have the same URL.
    foreach ($media_entities as $media) {
      $this->assertEquals($url, $media->get('field_media_oembed_social')->value);
    }
  }
}

==> media_remote_social-1.1.0.tar.gz: media_remote_social/tests/src/Kernel/RemoteSocialErrorHandlingTest.php <==
<?php

namespace Drupal\Tests\media_remote_social\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\media\Entity\Media;
use Drupal\media_remote_social_test\ResourceMiddleware;

/**
 * Tests error handling and edge cases for remote social media.
 *
 * @group media_remote_social
 */
class RemoteSocialErrorHandlingTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'file',
    'image',
    'media',
    'media_remote_social',
    'media_remote_social_test',
    'system',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig('media');
    $this->installConfig('media_remote_social');
    $this->installEntitySchema('file');
    $this->installEntitySchema('media');
    $this->installEntitySchema('user');
    $this->installSchema('file', ['file_usage']);
  }

  /**
   * Tests handling of invalid URLs.
   */
  public function testInvalidUrls(): void {
    $invalid_urls = [
      'not-a-url',
      'https://example.com/not-social-media',
      'https://youtube.com/watch?v=dQw4w9WgXcQ', // Not a supported provider
      'ftp://facebook.com/post/123', // Wrong protocol
      '', // Empty string
    ];

    foreach ($invalid_urls as $url) {
      try {
        $media = Media::create([
          'bundle' => 'remote_social',
          'name' => 'Test Invalid URL',
          'field_media_oembed_social' => $url,
        ]);
        $media->save();
        
        // If we get here without an exception, check that the media is created
        // but the source field validation should prevent invalid URLs.
        $violations = $media->get('field_media_oembed_social')->validate();
        $this->assertGreaterThan(0, $violations->count(), 'Invalid URL should produce validation errors: ' . $url);
      }
      catch (\Exception $e) {
        // Expected behavior for invalid URLs
        $this->assertTrue(TRUE, 'Exception thrown for invalid URL as expected: ' . $url);
      }
    }
  }

  /**
   * Tests handling of malformed oEmbed responses.
   */
  public function testMalformedOembedResponse(): void {
    $url = 'https://www.facebook.com/test/posts/12345/';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    // Create a temporary file with malformed JSON.
    $malformed_json = '{"version": "1.0", "type": "rich", invalid json here}';
    $temp_file = tempnam(sys_get_temp_dir(), 'malformed_oembed');
    file_put_contents($temp_file, $malformed_json);
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => $temp_file,
    ]);
    
    try {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // If save succeeds, the media should still be created but might have limited metadata.
      $this->assertInstanceOf(Media::class, $media);
    }
    catch (\Exception $e) {
      // This is also acceptable behavior.
      $this->assertStringContainsString('oEmbed', $e->getMessage());
    }
    finally {
      unlink($temp_file);
    }
  }

  /**
   * Tests handling of missing required oEmbed fields.
   */
  public function testMissingOembedFields(): void {
    $url = 'https://www.instagram.com/p/TEST123/';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    // Create oEmbed response with missing required fields.
    $incomplete_response = [
      'version' => '1.0',
      'type' => 'rich',
      // Missing: html, width, provider_name, etc.
    ];
    
    $temp_file = tempnam(sys_get_temp_dir(), 'incomplete_oembed');
    file_put_contents($temp_file, json_encode($incomplete_response));
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => $temp_file,
    ]);
    
    try {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // Check that the media was created even with incomplete data.
      $this->assertInstanceOf(Media::class, $media);
      
      // The media name should fall back to a default if title is missing.
      $this->assertNotEmpty($media->getName());
    }
    catch (\Exception $e) {
      // Some missing fields might cause exceptions.
      $this->assertTrue(strpos($e->getMessage(), 'oEmbed') !== FALSE || strpos($e->getMessage(), 'required') !== FALSE);
    }
    finally {
      unlink($temp_file);
    }
  }

  /**
   * Tests that unsupported providers are rejected.
   */
  public function testUnsupportedProviders(): void {
    // These URLs are valid oEmbed URLs but not in our supported list.
    $unsupported_urls = [
      'https://www.youtube.com/watch?v=dQw4w9WgXcQ',
      'https://vimeo.com/123456789',
      'https://soundcloud.com/artist/track',
    ];
    
    foreach ($unsupported_urls as $url) {
      $media = Media::create([
        'bundle' => 'remote_social',
        'field_media_oembed_social' => $url,
      ]);
      
      // Validate the field.
      $violations = $media->get('field_media_oembed_social')->validate();
      
      // There should be violations for unsupported providers.
      $this->assertGreaterThan(0, $violations->count(), 'Unsupported provider should produce validation errors: ' . $url);
      
      $has_provider_error = FALSE;
      foreach ($violations as $violation) {
        if (strpos($violation->getMessage(), 'provider') !== FALSE || strpos($violation->getMessage(), 'supported') !== FALSE) {
          $has_provider_error = TRUE;
          break;
        }
      }
      
      $this->assertTrue($has_provider_error, 'Validation should mention unsupported provider for: ' . $url);
    }
  }

  /**
   * Tests edge cases with special characters in URLs.
   */
  public function testSpecialCharacterUrls(): void {
    $special_urls = [
      'https://www.facebook.com/test%20user/posts/123456/', // URL encoded spaces
      'https://www.facebook.com/user/posts/123456789?ref=share', // Query parameters
      'https://www.instagram.com/p/ABC-123_DEF/', // Special characters in ID
    ];
    
    foreach ($special_urls as $url) {
      $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
      
      // Use the appropriate fixture based on the provider.
      $provider = 'facebook';
      if (strpos($url, 'instagram.com') !== FALSE) {
        $provider = 'instagram';
      }
      
      $fixture_file = __DIR__ . '/../../fixtures/' . $provider . '.json';
      
      $state = $this->container->get('state');
      $state->set(ResourceMiddleware::class, [
        $resource_url => $fixture_file,
      ]);
      
      $media = Media::create([
        'bundle' => 'remote_social',
        'name' => 'Test Special Characters',
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      
      // The media should be created successfully.
      $this->assertInstanceOf(Media::class, $media);
      $this->assertEquals($url, $media->get('field_media_oembed_social')->value);
    }
  }

  /**
   * Tests concurrent media creation with the same URL.
   */
  public function testConcurrentMediaCreation(): void {
    $url = 'https://www.facebook.com/page/posts/123456789';
    $resource_url = $this->container->get('media.oembed.url_resolver')->getResourceUrl($url);
    
    $state = $this->container->get('state');
    $state->set(ResourceMiddleware::class, [
      $resource_url => __DIR__ . '/../../fixtures/facebook.json',
    ]);
    
    // Create multiple media entities with the same URL.
    $media_entities = [];
    for ($i = 0; $i < 3; $i++) {
      $media = Media::create([
        'bundle' => 'remote_social',
        'name' => 'Test Facebook ' . $i,
        'field_media_oembed_social' => $url,
      ]);
      $media->save();
      $media_entities[] = $media;
    }
    
    // All media entities should be created successfully.
    $this->assertCount(3, $media_entities);
    
    // Each should have the same URL.
    foreach ($media_entities as $media) {
      $this->assertEquals($url, $media->get('field_media_oembed_social')->value);
    }
  }
}
